@@ -1,18 +1,26 @@
 #ifndef IPC_H
 #define IPC_H
 #include <syscall.h>
+#include <bits/wordsize.h>
+
+#if __WORDSIZE == 32 || defined __alpha__ || defined __mips__
+# define __IPC_64	0x100
+#else
+# define __IPC_64	0x0
+#endif
 
 #ifdef __NR_ipc
 
 /* The actual system call: all functions are multiplexed by this.  */
-extern int __syscall_ipc __P((int __call, int __first, int __second,
-					  int __third, void *__ptr));
+extern int __syscall_ipc (unsigned int __call, long __first, long __second,
+					  long __third, void *__ptr, void *__fifth) attribute_hidden;
 
 
 /* The codes for the functions to use the multiplexer `__syscall_ipc'.  */
 #define IPCOP_semop	 1
 #define IPCOP_semget	 2
 #define IPCOP_semctl	 3
+#define IPCOP_semtimedop 4
 #define IPCOP_msgsnd	11
 #define IPCOP_msgrcv	12
 #define IPCOP_msgget	13
