@@ -1,4 +1,5 @@
-/* Copyright (C) 1991, 93, 95, 96, 97, 2000 Free Software Foundation, Inc.
+/* Copyright (C) 1991,1993,1995-1997,2000
+   Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -27,17 +28,95 @@
 #include <stdlib.h>
 #include <time.h>
 #include <unistd.h>
+#include <sys/syscall.h>
 #include <sys/sysinfo.h>
 #include <sys/types.h>
+#include <sys/param.h>
+#ifdef __UCLIBC_HAS_REGEX__
 #include <regex.h>
+#endif
+#ifdef __UCLIBC_HAS_THREADS_NATIVE__
+#include <sysdep.h>
+#endif
+#include <sys/resource.h>
+#include <string.h>
+#include <dirent.h>
+#include "internal/parse_config.h"
+
+static int nprocessors_onln(void)
+{
+	char **l = NULL;
+	parser_t *p = config_open("/proc/stat");
+	int ret = 0;
+
+	if (p) {
+		while (config_read(p, &l, 2, 1, " ", 0))
+			if (l[0][0] == 'c'
+				&& l[0][1] == 'p'
+				&& l[0][2] == 'u'
+				&& isdigit(l[0][3]))
+				++ret;
+	} else if ((p = config_open("/proc/cpuinfo"))) {
+#if defined __sparc__
+		while (config_read(p, &l, 2, 2, "\0:", PARSE_NORMAL))
+			if (strncmp("ncpus active", l[0], 12) == 0) {
+				ret = atoi(l[1]);
+				break;
+			}
+#else
+		while (config_read(p, &l, 2, 2, "\0:\t", PARSE_NORMAL))
+			if (strcmp("processor", l[0]) == 0)
+				++ret;
+#endif
+	}
+	config_close(p);
+	return ret != 0 ? ret : 1;
+}
+
+#if defined __UCLIBC__ && !defined __UCLIBC_HAS_LFS__
+# define readdir64 readdir
+# define dirent64 dirent
+#endif
+static int nprocessors_conf(void)
+{
+	int ret = 0;
+	DIR *dir = opendir("/sys/devices/system/cpu");
+
+	if (dir) {
+		struct dirent64 *dp;
+
+		while ((dp = readdir64(dir))) {
+			if (dp->d_type == DT_DIR
+				&& dp->d_name[0] == 'c'
+				&& dp->d_name[1] == 'p'
+				&& dp->d_name[2] == 'u'
+				&& isdigit(dp->d_name[3]))
+				++ret;
+		}
+		closedir(dir);
+	} else
+	{
+#if defined __sparc__
+		char **l = NULL;
+		parser_t *p = config_open("/proc/stat");
+		while (config_read(p, &l, 2, 2, "\0:", PARSE_NORMAL))
+			if (strncmp("ncpus probed", l[0], 13) == 0) {
+				ret = atoi(l[1]);
+				break;
+			}
+		config_close(p);
+#else
+		ret = nprocessors_onln();
+#endif
+	}
+	return ret != 0 ? ret : 1;
+}
+
 
 #ifndef __UCLIBC_CLK_TCK_CONST
 #error __UCLIBC_CLK_TCK_CONST not defined!
 #endif
 
-extern int getpagesize (void);
-extern int getdtablesize (void);
-
 /***********************************************************************/
 /*
  * Manuel Novoa III        Jan 2001
@@ -47,7 +126,6 @@ extern int getdtablesize (void);
  * repeats and moving to a table-based implementation, we generate 283
  * bytes on i386 (-Os -fomit-frame-pointer).
  */
-#ifndef _UCLIBC_GENERATE_SYSCONF_MAIN
 
 #ifdef _UCLIBC_GENERATE_SYSCONF_ARCH
 /*
@@ -69,9 +147,15 @@ extern int getdtablesize (void);
 #define RETURN_FUNCTION(f) return f;
 #endif /* _UCLIBC_GENERATE_SYSCONF_ARCH */
 
+/* Legacy value of ARG_MAX.  The macro is now not defined since the
+   actual value varies based on the stack size.  */
+#define legacy_ARG_MAX 131072
+
 /* Get the value of the system variable NAME.  */
-long int __sysconf(int name)
+long int sysconf(int name)
 {
+  struct rlimit rlimit;
+
   switch (name)
     {
     default:
@@ -79,10 +163,13 @@ long int __sysconf(int name)
       return -1;
 
     case _SC_ARG_MAX:
-#ifdef	ARG_MAX
+      /* Use getrlimit to get the stack limit.  */
+      if (getrlimit (RLIMIT_STACK, &rlimit) == 0)
+          return MAX (legacy_ARG_MAX, rlimit.rlim_cur / 4);
+#if defined ARG_MAX
       return ARG_MAX;
 #else
-      RETURN_NEG_1;
+      return legacy_ARG_MAX;
 #endif
 
     case _SC_CHILD_MAX:
@@ -104,11 +191,7 @@ long int __sysconf(int name)
 #endif
 
     case _SC_OPEN_MAX:
-#if 1
       RETURN_FUNCTION(getdtablesize());
-#else
-      RETURN_NEG_1;
-#endif
 
     case _SC_STREAM_MAX:
 #ifdef	STREAM_MAX
@@ -118,12 +201,7 @@ long int __sysconf(int name)
 #endif
 
     case _SC_TZNAME_MAX:
-#if 0
-      RETURN_FUNCTION(tzname_max ());
-#else
-/*       RETURN_NEG_1; */
       return _POSIX_TZNAME_MAX;
-#endif
 
     case _SC_JOB_CONTROL:
 #ifdef	_POSIX_JOB_CONTROL
@@ -560,46 +638,37 @@ long int __sysconf(int name)
 
       /* POSIX 1003.1c (POSIX Threads).  */
     case _SC_THREADS:
-#ifdef	_POSIX_THREADS
+#ifdef __UCLIBC_HAS_THREADS__
       return 1;
 #else
       RETURN_NEG_1;
 #endif
 
     case _SC_THREAD_SAFE_FUNCTIONS:
-#ifdef	_POSIX_THREAD_SAFE_FUNCTIONS
+#ifdef __UCLIBC_HAS_THREADS__
       return 1;
 #else
       RETURN_NEG_1;
 #endif
 
+/* If you change these, also change libc/pwd_grp/pwd_grp.c to match */
     case _SC_GETGR_R_SIZE_MAX:
-#ifdef	NSS_BUFLEN_GROUP
-      return NSS_BUFLEN_GROUP;
-#else
-      RETURN_NEG_1;
-#endif
+      return __UCLIBC_GRP_BUFFER_SIZE__;
 
     case _SC_GETPW_R_SIZE_MAX:
-#ifdef	NSS_BUFLEN_PASSWD
-      return NSS_BUFLEN_PASSWD;
-#else
-      RETURN_NEG_1;
-#endif
+      return __UCLIBC_PWD_BUFFER_SIZE__;
 
+/* getlogin() is a worthless interface.  In uClibc we let the user specify
+ * whatever they want via the LOGNAME environment variable, or we return NULL
+ * if getenv() fails to find anything.  So this is merely how large a env
+ * variable can be.  Lets use 256 */
     case _SC_LOGIN_NAME_MAX:
-#ifdef	_POSIX_LOGIN_NAME_MAX
-      return _POSIX_LOGIN_NAME_MAX;
-#else
-      RETURN_NEG_1;
-#endif
+      return 256;
 
+/* If you change this, also change _SC_TTY_NAME_MAX in libc/unistd/sysconf.c */
+#define TTYNAME_BUFLEN		32
     case _SC_TTY_NAME_MAX:
-#ifdef	_POSIX_TTY_NAME_MAX
-      return _POSIX_TTY_NAME_MAX;
-#else
-      RETURN_NEG_1;
-#endif
+      return TTYNAME_BUFLEN;
 
     case _SC_THREAD_DESTRUCTOR_ITERATIONS:
 #ifdef	_POSIX_THREAD_DESTRUCTOR_ITERATIONS
@@ -672,18 +741,10 @@ long int __sysconf(int name)
 #endif
 
     case _SC_NPROCESSORS_CONF:
-#if 0
-      RETURN_FUNCTION(get_nprocs_conf());
-#else
-      RETURN_NEG_1;
-#endif
+      RETURN_FUNCTION(nprocessors_conf());
 
     case _SC_NPROCESSORS_ONLN:
-#if 0
-      RETURN_FUNCTION(get_nprocs());
-#else
-      RETURN_NEG_1;
-#endif
+      RETURN_FUNCTION(nprocessors_onln());
 
     case _SC_PHYS_PAGES:
 #if 0
@@ -878,6 +939,30 @@ long int __sysconf(int name)
 #else
       RETURN_NEG_1;
 #endif
+    case _SC_V7_ILP32_OFF32:
+#ifdef _POSIX_V7_ILP32_OFF32
+      return _POSIX_V7_ILP32_OFF32;
+#else
+      RETURN_NEG_1;
+#endif
+    case _SC_V7_ILP32_OFFBIG:
+#ifdef _POSIX_V7_ILP32_OFFBIG
+      return _POSIX_V7_ILP32_OFFBIG;
+#else
+      RETURN_NEG_1;
+#endif
+    case _SC_V7_LP64_OFF64:
+#ifdef _POSIX_V7_LP64_OFF64
+      return _POSIX_V7_LP64_OFF64;
+#else
+      RETURN_NEG_1;
+#endif
+    case _SC_V7_LPBIG_OFFBIG:
+#ifdef _POSIX_V7_LPBIG_OFFBIG
+      return _POSIX_V7_LPBIG_OFFBIG;
+#else
+      RETURN_NEG_1;
+#endif
 
     case _SC_XOPEN_LEGACY:
       return _XOPEN_LEGACY;
@@ -893,169 +978,34 @@ long int __sysconf(int name)
       return _XOPEN_REALTIME_THREADS;
 #else
       RETURN_NEG_1;
+#endif
+
+    case _SC_MONOTONIC_CLOCK:
+#ifdef __NR_clock_getres
+    /* Check using the clock_getres system call.  */
+# ifdef __UCLIBC_HAS_THREADS_NATIVE__
+    {
+      struct timespec ts;
+      INTERNAL_SYSCALL_DECL (err);
+      int r;
+      r = INTERNAL_SYSCALL (clock_getres, err, 2, CLOCK_MONOTONIC, &ts);
+      return INTERNAL_SYSCALL_ERROR_P (r, err) ? -1 : _POSIX_VERSION;
+    }
+# elif defined __UCLIBC_HAS_REALTIME__
+      if (clock_getres(CLOCK_MONOTONIC, NULL) >= 0)
+        return _POSIX_VERSION;
+# endif
+#endif
+      RETURN_NEG_1;
+
+#ifdef __UCLIBC_HAS_THREADS_NATIVE__
+    case _SC_THREAD_CPUTIME:
+# if _POSIX_THREAD_CPUTIME > 0
+      return _POSIX_THREAD_CPUTIME;
+# else
+      RETURN_NEG_1;
+# endif
 #endif
     }
 }
-weak_alias(__sysconf, sysconf);
-
-#endif /* _UCLIBC_GENERATE_SYSCONF_MAIN */
-/***********************************************************************/
-#ifdef _UCLIBC_GENERATE_SYSCONF_MAIN
-
-static long int ret_vals[_UCLIBC_SYSCONF_NUM_VALID_ARGS];
-
-static const char *type_str[] = {
-	"char", "short", "int", "long"
-};
-
-static const char *type_fmt[] = { " %4ld", " %6ld", " %8ld", " %8ld" };
-static const int type_mod[] = { 13, 9, 6, 6 };
-
-static int find_or_add_in_table(int index, long int val, int *table, 
-								int *num_in_table, int add_flag)
-{
-	int i;
-
-	for (i=0 ; i<*num_in_table ; i++) {
-		if (ret_vals[table[i]] == val) {
-			return i;
-		}
-	}
-	if (add_flag) {
-		table[(*num_in_table)++] = index;
-		return i;
-	} else {
-		return -1;
-	}
-}
-
-
-int main(void)
-{
-	long int r;
-	int type_table[5][_UCLIBC_SYSCONF_NUM_VALID_ARGS];
-	int ret_type[_UCLIBC_SYSCONF_NUM_VALID_ARGS];
-	int num_type[5];
-	int i, j, k, code;
-
-	for (i=0 ; i<5 ; i++) {
-		num_type[i] = 0;
-	}
-
-	for (i=0; i<_UCLIBC_SYSCONF_NUM_VALID_ARGS ; i++) {
-		__set_errno(0);
-		r = ret_vals[i] = sysconf(i);
-		switch(errno) {
-			case EINVAL:		/* we're missing a case! */
-				fprintf(stderr,"sysconf.c is broken! case %d missing!\n", i);
-				return EXIT_FAILURE;
-			case EISNAM:		/* function */
-				find_or_add_in_table(i,r,type_table[4],num_type+4,1);
-				ret_type[i] = 4;
-				break;
-			case ENOSYS:		/* defaults to -1 */
-				/* why does this break for shared???? */
-				fprintf(stderr,"gen_sysconf advisory --"
-						"case %d defaults to -1\n", i);
-				/* fall through */
-			case 0:
-				if ((r>=CHAR_MIN) && (r<=CHAR_MAX)) {
-					ret_type[i] = 0;
-					find_or_add_in_table(i,r,type_table[0],num_type+0,1);
-				} else if ((r>=SHRT_MIN) && (r<=SHRT_MAX)) {
-					ret_type[i] = 1;
-					find_or_add_in_table(i,r,type_table[1],num_type+1,1);
-				} else if ((r>=INT_MIN) && (r<=INT_MAX)) {
-					ret_type[i] = 2;
-					find_or_add_in_table(i,r,type_table[2],num_type+2,1);
-				} else {
-					ret_type[i] = 3;
-					find_or_add_in_table(i,r,type_table[3],num_type+3,1);
-				}
-				break;
-			default:
-				fprintf(stderr,"sysconf.c is broken! errno = %d!\n", errno);
-				break;
-		}
-	}
-
-	printf("#include <errno.h>\n#include <unistd.h>\n#include <limits.h>\n\n");
-
-	printf("static const unsigned char index[%d] = {",
-		   _UCLIBC_SYSCONF_NUM_VALID_ARGS);
-	for (i=0 ; i<_UCLIBC_SYSCONF_NUM_VALID_ARGS ; i++) {
-		if (i) printf(",");
-		if (i%15 == 0) printf("\n");
-		code = 0;
-		for (j=0 ; j<4 ; j++) {
-			k = find_or_add_in_table(i,ret_vals[i],type_table[j],num_type+j,0);
-			if (k>=0) {
-				code += k;
-				break;
-			}
-			code += num_type[j];
-		}
-		printf(" %3d", code);
-	}
-	printf("\n};\n\n");
-
-	for (j=0 ; j < 4 ; j++) {
-		if (num_type[j]) {
-			printf("static const %s %s_vals[%d] = {",
-				   type_str[j], type_str[j], num_type[j]);
-			for (i = 0 ; i<num_type[j] ; i++) {
-				if (i) printf(",");
-				if (i%13 == 0) printf("\n");
-				if (ret_vals[type_table[j][i]] == INT_MIN) {
-					printf("%12s","INT_MIN");
-				} else {
-					printf(type_fmt[j], ret_vals[type_table[j][i]]);
-				}
-
-			}
-			printf("\n};\n\n");
-		}
-	}
-
-	printf("long int sysconf(int name)\n{\n\tint i;\n\n"
-		   "\tif ((name < 0) || (name >= %d)) {\n"
-		   "\t\terrno=EINVAL;\n"
-		   "\t\treturn -1;\n"
-		   "\t}\n\n", _UCLIBC_SYSCONF_NUM_VALID_ARGS);
-
-	printf("\ti = index[name];\n\n");
-	k = 0;
-	for (i=0 ; i<4 ; i++) {
-		if (num_type[i]) {
-			if (k>0) {
-				printf("\ti -= %d;\n", k);
-			}
-			printf("\tif (i < %d) {\n"
-				   "\t\treturn %s_vals[i];\n"
-				   "\t}\n\n",
-				   num_type[i], type_str[i]);
-			k = num_type[i];
-		}
-	}
-
-	if (num_type[4]) {
-		if (k>0) {
-			printf("\ti -= %d;\n", k);
-		}
-		printf("\tswitch(i) {\n");
-		for (i = 0 ; i<num_type[4] ; i++) {
-			printf("\t\tcase %d:\n"
-				   "\t\t\treturn %s;\n",
-				   i, (const char *)ret_vals[type_table[4][i]]);
-		}
-		printf("\t}\n\n");
-	}
-
-	printf("\treturn -1;\n"
-		   "}\n");
-
-
-	return EXIT_SUCCESS;
-}
-/***********************************************************************/
-#endif
+libc_hidden_def(sysconf)
